package org.esupportail.cas.server.handlers.database;

import org.dom4j.Element;
import org.esupportail.cas.server.util.MisconfiguredHandlerException;
import org.esupportail.cas.server.util.crypt.Crypt;
import org.esupportail.cas.server.util.log.Log;

/**
 * This class implements a query database handler class.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 * @author Arunas Stockus <arunas.stockus at univ-lr.fr>
 */
public class QueryDatabaseHandler extends DatabaseHandler {

	/**
	 * This token is replaced by the users' login when executing queries on the database. 
	 */
	static final String SQL_LOGIN_TOKEN = "%u";
	
	/**
	 * the string used to query the database. This query is supposed 
	 * to return a single VARCHAR column.
	 */
	private String sqlQuery;
	/**
	 * the encryption used for stored passwords.
	 */
	private String encryption;
	/**
	 * the username to use to bind to the database.
	 */
	private String bindUsername;
	/**
	 * the password to use to bind to the database.
	 */
	private String bindPassword;
	
	/**
	 * Constructor.
	 *
	 * @param handlerElement the XML element that declares the handler 
	 * in the configuration file
	 * @param configDebug debugging mode of the global configuration
	 * @throws Exception Exception
	 */
	public QueryDatabaseHandler(
			final Element handlerElement, 
			final Boolean configDebug) throws Exception {
		super(handlerElement, configDebug);
		traceBegin();

		sqlQuery = readSqlQueryFromConfig();
		trace("sql_query = " + sqlQuery);
		if (sqlQuery.indexOf(SQL_LOGIN_TOKEN) < 0) {
			Log.warn("The SQL query should contain at least one \"" 
					+ SQL_LOGIN_TOKEN 
					+ "\"; assuming the user knows what he does.");
		}

		bindUsername = getConfigSubElementContent("bind_username", false/*not needed*/);
		if (bindUsername.equals("")) {
			trace("An anonymous connection to the database will be used.");
			bindPassword = "";
		} else {
			bindPassword = getConfigSubElementContent("bind_password", false/*not needed*/);
		}
		trace("bind_username = " + bindUsername);
		trace("bind_password = " + bindPassword);

		encryption = getConfigSubElementContent("encryption", false/*not needed*/);
		if (encryption.equals("")) {
			encryption = "md5";
		}
		if (!Crypt.isEncryptionSupported(encryption)) {
			traceThrow(new MisconfiguredHandlerException(
					"Encryption \"" 
					+ encryption 
					+ "\" is not supported."));
		}
		trace("encryption = " + encryption);
		if (encryption.equals("plain")) {
			Log.warn("Passwords should be encrypted. Be sure to keep the database out of danger!");
		}

		// add the database servers
		addServers(true/*serverElementNeeded*/, getClass().getPackage().getName() + ".QueryDatabaseServer");

		traceEnd();
	}
	
	/**
	 * Read the SQL query from the configuration.
	 * @return a String.
	 * @throws Exception Exception
	 */
	protected String readSqlQueryFromConfig() throws Exception {
		traceBegin();
		String query = getConfigSubElementContent("sql_query", true/*needed*/);
	    // Remove all tailing semi-colons (they hurt some JDBC drivers).
		boolean semiColonFound = false;
		while (query.endsWith(";")) {
	      query.substring(0, query.length() - 1).trim();
	      semiColonFound = true;
	    }
		if (semiColonFound) {
			Log.warn("Trailing semi-colons are not allowed in SQL queries, they will be ignored.");
		}
		traceEnd(query);
		return query;
	}
	
	/**
	 * Returns the SQL expression that must be executed to
	 * retrieve the password of the user with the given name.
	 * @param username the user's id
	 * @return a String.
	 */
	final String getUserSqlQuery(final String username) {		
		return sqlQuery.replaceAll(SQL_LOGIN_TOKEN, username);
}	

	/**
	 * Retrieve the encryption used to store passwords.
	 *
	 * @return a String Object.
	 */
	final String getEncryption() {
		return encryption;
	}
	
	/**
	 * Retrieve the username to use to bind to the database.
	 *
	 * @return a String Object.
	 */
	final String getBindUsername() {
		return bindUsername;
	}
	
	/**
	 * Retrieve the password to use to bind to the database.
	 *
	 * @return a String Object.
	 */
	final String getBindPassword() {
		return bindPassword;
	}
	
}

