package org.esupportail.cas.server.handlers.database;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.dom4j.Element;
import org.esupportail.cas.server.util.RedundantHandler;
import org.esupportail.cas.server.util.crypt.Crypt;
import org.esupportail.cas.server.util.log.Log;

/**
 * This class also implements a database server class, which can
 * authenticate users by searching into a database. It is used by 
 * SearchDatabaseHandler.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 * @author Jean-Baptiste Daniel <danielj at users.sourceforge.net>
 * @author Arunas Stockus <arunas.stockus at univ-lr.fr>
 */
public final class QueryDatabaseServer extends DatabaseServer {
	
	/**
	 * Constructor.
	 *
	 * @param handlerDebug debugging mode of the handler
	 * @param handler      the handler the server will be used by
	 * @param serverElement the XML element that declares the server 
	 * @throws Exception Exception
	 */
	public QueryDatabaseServer(
			final Boolean handlerDebug,
			final RedundantHandler handler,
			final Element serverElement) throws Exception {
		super(handlerDebug, handler, serverElement);
		traceBegin();
		traceEnd();
	}
	
	/**
	 * Try to authenticate a user (by searching into the handler's database).
	 *
	 * @param username the user's name
	 * @param password the user's password
	 *
	 * @return Server.AUTHENTICATE_SUCCESS, Server.AUTHENTICATE_NOAUTH
	 * or Server.AUTHENTICATE_FAILURE.
	 */	
	public int authenticate(
			final String username,
			final String password) {
		Connection connection = null;
		traceBegin();
		
		if (username.indexOf('\'') != -1) {
			Log.info("Username \"" + username + "\" contains a single quote, this could be an attack.");
			trace("Usernames containing single quotes are rejected.");
			traceEnd("AUTHENTICATE_NOAUTH");
			return AUTHENTICATE_NOAUTH;
		}
		
		QueryDatabaseHandler handler = (QueryDatabaseHandler) getHandler();
		
		try {
			
			connection = connect(handler.getBindUsername(), handler.getBindPassword());
			switch (getConnectError()) {
				case CONNECT_NOAUTH:
				case CONNECT_FAILURE:
					trace("Connection failed.");
					traceEnd("AUTHENTICATE_FAILURE");
					return AUTHENTICATE_FAILURE;
				default: //CONNECT_SUCCESS
					break;
			}
			
			trace("Create an SQL statement...");
			Statement statement = connection.createStatement();
			
			String sqlQuery = handler.getUserSqlQuery(username);
			
			trace("Execute the query (" + sqlQuery + ")...");
			ResultSet statementResult = statement.executeQuery(sqlQuery);
			
			if (statementResult.next()) { 
				
				String encryption = handler.getEncryption();
				trace("Username found, checking password (" + encryption + ")...");
				// now for each result, encrypt the password (if needed) and compare
				String encryptedPassword = statementResult.getString(1);
				if (Crypt.match(encryption, password, encryptedPassword)) {
					// username/password matches
					trace("Password matches.");
					trace("Closing the connection...");
					connection.close();
					traceEnd("AUTHENTICATE_SUCCESS");
					return AUTHENTICATE_SUCCESS;
				} else {
					trace("Username found but password does not match.");
				}
			}
			// no match found
			trace("Search failed.");
			trace("Closing the connection...");
			connection.close();
			traceEnd("AUTHENTICATE_NOAUTH");
			return AUTHENTICATE_NOAUTH;
			
		} catch (Exception e) {
			// something went wrong
			trace("Search failure: " + e.toString());
			try {
				trace("Closing the connection...");
				connection.close();
			} catch (SQLException e1) {
				Log.warn("Connection could not be closed.");
			}
			traceEnd("AUTHENTICATE_FAILURE"); 
			return AUTHENTICATE_FAILURE; 
		}
		
	}

}
