package org.esupportail.cas.server.util;

import org.dom4j.Element;
import org.esupportail.cas.server.util.log.Debug;

/**
 * This class implements a basic handler, without any property. 
 * This abstract class is inherited by any handler.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
public abstract class BasicHandler extends Debug {
	
	/**
	 * The value return by authenticate() when the authentication
	 * was successfull.
	 */
	public static final int SUCCEEDED = 1;
	/**
	 * The value return by authenticate() when the authentication
	 * failed but the next handler should be tried.
	 */
	protected static final int FAILED_CONTINUE = 2;
	/**
	 * The value return by authenticate() when the authentication
	 * failed and the authentication process should stop.
	 */
	public static final int FAILED_STOP = 3;

	/**
	 * The XML element that represents the configuration of the handler
	 * in the configuration file.
	 */
	private Element configElement;

	
	/**
	 * Retrieve the XML element the represents the configuration of the handler
	 * in the configuration file.
	 * 
	 * @return an XML element.
	 */
	protected final Element getConfigElement() {
		return configElement;
	}

	/**
	 * Constructor.
	 *
	 * @param handlerElement the XML element that declares the handler 
	 * in the configuration file
	 * @param configDebug debugging mode of the global configuration
	 */
	protected BasicHandler(
			final Element handlerElement, 
			final Boolean configDebug) {
		super(elementDebugValue(handlerElement, configDebug.booleanValue()));
		traceBegin();
		configElement = handlerElement.element("config");
		traceEnd();
	}
	
	/**
	 * Check the "config" XML element if needed.
	 *
	 * @param configElementNeeded true if a config element is needed
	 * @throws Exception Exception
	 */	
	protected final void checkConfigElement(final boolean configElementNeeded) throws Exception {
		traceBegin();
		if (configElementNeeded && (getConfigElement() == null)) {
			traceThrow(new MisconfiguredHandlerException(
					"A \"config\" element is needed by \"" 
					+ getClass().getName() 
					+ "\" handlers."));
		}
		traceEnd();
	}
	
	/**
	 * Tries to Authenticate a user by accessing all the servers..
	 *
	 * @param username the username to authenticate
	 * @param password the corresponding password
	 *
	 * @return SUCCEDED on success, FAILED_CONTINUE or FAILED_STOP otherwise.
	 */
	public abstract int authenticate(
			final String username,
			final String password);

	/**
	 * Retrieve the content of a config sub-element.
	 * 
	 * @param elementName the name of the XML element to look for
	 * @param needed false if the element can be absent or empty, true otherwise
	 * @return a String
	 * @throws Exception Exception
	 */
	protected final String getConfigSubElementContent(
			final String elementName, 
			final boolean needed) throws Exception {
		Element element = getConfigElement().element(elementName);
		String str;
		if (element == null) {
			str = "";
		} else {
			str = element.getTextTrim();
		}
		if (needed && str.equals("")) {
			traceThrow(new MisconfiguredHandlerException(
					"A non empty nested \"" + elementName + "\" element is needed to configure \"" 
					+ getClass().getName() 
					+ "\" handlers."
					));
		}
		return str;
	}

	/**
	 * Tell if a config sub-element is present.
	 * 
	 * @param elementName the name of the XML element to look for
	 * @return true if the element is present, false otherwise
	 * @throws Exception Exception
	 */
	protected final boolean hasConfigSubElement(
			final String elementName) throws Exception {
		Element element = getConfigElement().element(elementName);
		return (element != null);
	}

}

